#! /usr/bin/env python

## Copyright 2000-2001 by LivingLogic AG, Bayreuth, Germany.
## Copyright 2000-2001 by Walter Drwald
##
## All Rights Reserved
##
## Permission to use, copy, modify, and distribute this software and its documentation
## for any purpose and without fee is hereby granted, provided that the above copyright
## notice appears in all copies and that both that copyright notice and this permission
## notice appear in supporting documentation, and that the name of Living Logic AG or
## the author not be used in advertising or publicity pertaining to distribution of the
## software without specific, written prior permission.
##
## LIVINGLOGIC AG AND THE AUTHOR DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
## INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
## LIVINGLOGIC AG OR THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
## DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
## IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
## IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""
A module that contains elements that simplify generating SQL statements.

Available classes are Insert, Update and Delete.

Creating an Insert statement simply works like this:
	query = sql.Insert("TABLENAME", {"FIELDNAME1": "'Value1'", "FIELDNAME2": 42, "FIELDNAME3": None})

print str(query) will give you:
	INSERT INTO TABLENAME (FIELDNAME1, FIELDNAME2, FIELDNAME3) VALUES('''Value1''', 42, NULL);

Executing the query can be done via:
	cursor.execute(str(query))
or via:
	query.execute(cursor)

Creating an Update statement works like this:
	query = sql.Update("TABLENAME", {"FIELDNAME1": "'Value1'"}, {"FIELDNAME2": 42, "FIELDNAME3": None})

The query generated from this object is:
	UPDATE TABLENAME SET FIELDNAME1='''Value1''' WHERE FIELDNAME2=42 AND FIELDNAME3 IS NULL;

WHERE clauses that are more involved than that are currently not possible,
and probably never will.
"""

__version__ = tuple(map(int, "$Revision: 1.2 $"[11:-2].split(".")))
# $Source: /data/cvsroot/LivingLogic/sql/sql.py,v $

import types

class Command:
	"""
	encapsulates an SQL command and provides a bunch of services for derived classes
	"""

	def _formatValue(self, value):
		t = type(value)
		if t is types.NoneType:
			return "NULL"
		elif t in (types.StringType, types.UnicodeType):
			return ("'" + value.replace("'", "''") + "'").encode("latin1")
		elif t in (types.IntType, types.LongType, types.FloatType):
			return str(value)
		else:
			raise ValueError, "unrecognised type for database field"

	def _formatField(self, name, value, format=0):
		"""
		format == 0: setting
		format == 1: testing
		format == 2: inserting
		"""
		if value is None:
			if format==0:
				return name + "=NULL"
			elif format==1:
				return name + " IS NULL"
			else:
				return "NULL"
		else:
			if format==0 or format==1:
				return name + "=" + self._formatValue(value)
			else:
				return self._formatValue(value)

	def _formatFields(self, fields, format=0):
		v = []
		for field in fields.keys():
			v.append(self._formatField(field, fields[field], format))
		if format==0:
			return ",".join(v)
		elif format==1:
			return " AND ".join(v)
		else:
			return ",".join(v)

	def execute(self, cursor):
		return cursor.execute(str(self))

class Insert(Command):
	"""
	An SQL insert command. Attributes are table, the table name,
	and set, the dictionary with the field names as keys and
	the field values as values. None will be treated as NULL.

	Convert the Insert object to the query string with str().
	"""
	def __init__(self, table, set):
		self.table = table
		self.set = set

	def __str__(self):
		v = []
		v.append("INSERT INTO ")
		v.append(self.table)
		v.append(" (")
		vv = []
		for field in self.set.keys():
			vv.append(field)
		v.append(",".join(vv))
		v.append(") VALUES (")
		v.append(self._formatFields(self.set, 2))
		v.append(");")
		return "".join(v)

class Update(Command):
	"""
	An SQL update command. Attributes are table, the table name,
	set, the dictionary with the field names as keys and
	the field values as values and where, a dictionary that
	maps field names to required field values. None will be treated as
	NULL for both set and where.

	Convert the Update object to the query string with str().
	"""
	def __init__(self, table, set, where):
		self.table = table
		self.set = set
		self.where = where

	def __str__(self):
		v = []
		v.append("UPDATE " + self.table + " SET ")
		v.append(self._formatFields(self.set, 0))
		if len(self.where.keys()):
			v.append(" WHERE ")
			v.append(self._formatFields(self.where, 1))
		v.append(";")
		return "".join(v)

class Delete(Command):
	"""
	An SQL delete command. Attributes are table, the table name,
	and where, a dictionary that map field names to field values.
	for the where clause. None will be treated as NULL.

	Convert the Delete object to the query string with str().
	"""
	def __init__(self, table, where):
		self.table = table
		self.where = where

	def __str__(self):
		v = []
		v.append("DELETE FROM " + self.table)
		if len(self.where.keys()):
			v.append(" WHERE ")
			v.append(self._formatFields(self.where, 1))
		v.append(";")
		return "".join(v)
